<?php
/**
 * Bereitstellung von Funktionen zum Abrauf der Viessmann-Daten
 * 
 * Zur Verwendung muß ein Konto unter https://developer.viessmann.com/start.html aktiv sein.
 *
 * @author Dieter Lang <info@lang-dieter.de> 
 * @version 0.0 11/02/2024 09:14
 * Klassenvariablen
 * @param InstallationID = eindeutige ID der Installation bei Viessmann - kann durch API-Abruf ermittlet werden
 * @param gatewaySerial = Seriennummer des verwendeten WLAN-Gateways - kann durch API-Abruf ermittelt werden
 * @param deviceID = eindeutige ID des verwendeten WLAN-Gateways - kann durch API-Abruf ermittelt werden
*/
//Falls aus dem Downloadserver meiner Homepage direkt angeklickt wurde, den User darauf hinweisen.
if (stristr(strtolower($_SERVER['SCRIPT_FILENAME']),"/var/www/html/media/files/")) {
  echo "<p>Sie können die Datei auf diesem Server nicht erfolgreich ausführen sondern nur mit rechter Maustaste herunterladen";
  echo "<br><a href='https://www.lang-dieter.de/media/files/raspberry-pi/viessmannapi/viessmann.zip'https://www.lang-dieter.de/media/files/raspberry-pi/viessmannapi/viessmann.zip</a>";
  echo "</p>";
  die("Weitere Infos auf <a href='https://www.lang-dieter.de/viessmman-api-heizungswerte-anzeigen.html'>https://www.lang-dieter.de</a>");
}

class viessmann {

	//API-Zugriffe
	public $autorizationTOKEN="";
	public $authorizeURL = "https://iam.viessmann.com/idp/v3/authorize";
	public $tokenURL = "https://iam.viessmann.com/idp/v3/token";

	//Folgende Angaben werden im Viessmann-Portal beim Gerät registriert:
	public $client_id = "46fef057a76db12180d32cb5f38dc357";
	public $callback_uri = "http://localhost:4200/";
	public $user = "info@lang-dieter.de"; // The same as used for Vicare 
	public $pwd = "bus8enyxKB6IJBNJ0Qiw"; // The same as used for Vicare 
	//Die nächsten beiden Zeilen wurden mit dem Tool PKCE Code-Generator erzeugt https://developer.pingidentity.com/en/tools/pkce-code-generator.html
	public $code_verif="J8qqF6603vjsKdkfZIIambswbBC5WowHFee790xhzPevxogtLpyWcKd8DJ14IUFhWz8ydz88f_ovzpNLtrNBXf2MZtBVu_4rcX5S9RxAlw1frUl3QyhHWrRHJ9MTSm1e";
	public $code_challenge="HM6tPamSLkf2QZ1Lw5V2lBMHbS_BPr8GBy0wPQQJN4o";
	public $code_challenge_medoth="S256";
	public $scope="IoT%20User";
	public $response_type="code";

	public $dateiname_api_token = "";
	public $installationId="";
	public $gatewaySerial="";
	public $deviceId="";
	
	//Mittels API abgefragte Daten
	public $api_data = "";
	

	/**
	 * Initialisierung der Klasse beim ersten Aufruf
	*/
	function __construct(){
		$this->dateiname_api_token = $_SERVER['DOCUMENT_ROOT']."/api_token.txt";
		//echo "<br>".__file__."<br> Zeile: ".__line__." this=".str_replace(chr(10),"<br>",print_r($this,true))."<br><br>";	

		//Autorsierungsantrag
		$this->autorizationTOKEN = $this->api_autorisierungsantrag();
		//echo "<br>".__file__."<br> Zeile: ".__line__.": this->autorizationTOKEN= ".$this->autorizationTOKEN;

		//Grundeinstellungen für Kommunikation einlesen
		$getURL = "https://api.viessmann.com/iot/v1/equipment/installations?includeGateways=true";
		$erg = $this->api_curl_abruf($getURL);
		//echo "<br>".__file__."<br> Zeile: ".__line__.": getURL=".$getURL."<br>erg=".str_replace(chr(10),"<br>",print_r($erg,true))."<br><br>";	
		//echo "<br>".__file__."<br> Zeile: ".__line__." erg=".str_replace(chr(10),"<br>",print_r($erg[0]->data[0]->gateways[0]->devices[0],true))."<br><br>";	
		$this->installationId=$erg[0]->data[0]->id;
		$this->gatewaySerial=$erg[0]->data[0]->gateways[0]->devices[0]->gatewaySerial;
		$this->deviceId=0;
		//echo "<br>".__file__." Zeile ".__line__.": viessmann=".str_replace(chr(10),"<br>",print_r($this,true))."<br><br>";	

	}

	/**
	 * api_curl_abruf - API-Abruf bei Viessmann durchführen
	 * 
	 * @param geturl string API-Abruf gemäss Dokumentation von Viessmann
	 * @param debug boolean True zeigt Werte innerhalb der Funktion an
	 * @return erg array 0=Antwort im Json-Format 1= Aufgetretene API-Fehler - bei Erfolg leer
	*/
	function api_curl_abruf($getURL,$debug = False){

		$headerRequest = array("Content-Type: application/x-www-form-urlencoded","Authorization: Bearer $this->autorizationTOKEN");

		$cURLConnection = curl_init();
		curl_setopt($cURLConnection, CURLOPT_USERAGENT, "Mozilla/5.0 (compatible; MSIE 7.01; Windows NT 5.0)");
		curl_setopt($cURLConnection, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($cURLConnection, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($cURLConnection, CURLOPT_FOLLOWLOCATION, 0);
		curl_setopt($cURLConnection, CURLOPT_RETURNTRANSFER,true);
		curl_setopt($cURLConnection, CURLOPT_HTTPHEADER, $headerRequest); 
		curl_setopt($cURLConnection, CURLOPT_CONNECTTIMEOUT, 130);
		curl_setopt($cURLConnection, CURLOPT_TIMEOUT, 130);
		curl_setopt($cURLConnection, CURLOPT_URL,$getURL);

		$apiResponse = curl_exec($cURLConnection);
		$apiErr = curl_error($cURLConnection);

		curl_close($cURLConnection);

		$jsonResponse = json_decode($apiResponse);
		if ($debug){
			echo "<br>".__file__."<br> Zeile: ".__line__.": ".str_replace(chr(10),"<br>",print_r($jsonResponse,true))."<br><br>";	
			echo "<br>".__file__."<br> Zeile: ".__line__." apiErr=".str_replace(chr(10),"<br>",print_r($apiErr,true))."<br><br>";	
		}
		return(array($jsonResponse,$apiErr));
	}

	/**
	 * api_autorierungsantrag - Schritt 1 Anforderung eines Autoriierungscodes - kann auch im Kundenportal manuell erstellt werden.
	 * @return api_token string Token der für Zugriff benötigt wird - zwei verschiende Gerät und aktuell Sitzung
	 */
	function api_autorisierungsantrag() {


		// Code parameters 
		//
		$url = "$this->authorizeURL?client_id=$this->client_id&code_challenge=$this->code_challenge&scope=IoT%20User&redirect_uri=$this->callback_uri&response_type=code";
		$header = array("Content-Type: application/x-www-form-urlencoded");

		$curloptions = array(
			CURLOPT_URL => $url,
			CURLOPT_HTTPHEADER => $header,
			CURLOPT_SSL_VERIFYPEER => false,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_USERPWD => "$this->user:$this->pwd",
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_POST => true,
		);

		// Call Curl Code 
		//
		$curl = curl_init();
		curl_setopt_array($curl, $curloptions);
		$response = curl_exec($curl);
		curl_close($curl);

		// Code Extraction 
		//
		$matches = array();
		$pattern = '/code=(.*)"/';
		//$test= preg_match_all($pattern, $response, $matches);
		//echo "<br>".__file__." Zeile ".__line__.": test=".$test."<br>response=".$response;

		if (preg_match_all($pattern, $response, $matches)) {
			$code = $matches[1][0];
		} else {
			exit(__file__." Zeile ".__line__.": Fehler beim Codeabruf"."\n");
		}

		// Token Settings 
		//
		$url = "$this->tokenURL?grant_type=authorization_code&code_verifier=$this->code_challenge&client_id=$this->client_id&redirect_uri=$this->callback_uri&code=$code";
		$header = array("Content-Type: application/x-www-form-urlencoded");

		$curloptions = array(
			CURLOPT_URL => $url,
			CURLOPT_HTTPHEADER => $header,
			CURLOPT_SSL_VERIFYPEER => false,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
			CURLOPT_POST => true,
		);

		// Call Curl Token 
		//
		$curl = curl_init();
		curl_setopt_array($curl, $curloptions);
		$response = curl_exec($curl);
		curl_close($curl);

		// Token extraction 
		//
		$json = json_decode($response, true);
		$token = $json['access_token'];

		return($token);

	}

	/**
	 * api_listfeatures - Liste aller verfügbaren API-Funktionen ermitteln
	 * @return api_liste string Array mit den jweiligen Bezeichnung des Feature und der notwendigne URL
	 */
	function api_listfeatures(){
		$getURL='https://api.viessmann.com/iot/v1/equipment/installations/'.$this->installationId.'/gateways/'.$this->gatewaySerial.'/devices/'.$this->deviceId.'/features'		;
		$erg = $this->api_curl_abruf($getURL);
		//echo "<br>".__file__."<br> Zeile: ".__line__.": ".$getURL."<br> erg=".str_replace(chr(10),"<br>",print_r($erg,true))."<br><br>";

		foreach($erg as $zeile => $inhalt){
			echo "<br>".__file__." Zeile: ".__line__.": Datenzeile ".$zeile;
			//echo "<br>".__file__." Zeile: ".__line__.": Inhalt".str_replace(chr(10),"<br>",print_r($inhalt,true));
			if (is_object($inhalt)){
				foreach($inhalt as $key => $data){
					//echo "<br>".__file__." Zeile: ".__line__.": key ".$key." => data=".str_replace(chr(10),"<br>",print_r($data,true));
					if ($key == "statusCode"){
						echo "<h2>Es ist ein Fehler aufgetreten:</h2>";
						echo "<a href=".$getURL."' target=new>".$getURL."</a>";
						echo "<br>".__file__." Zeile: ".__line__.": Inhalt".str_replace(chr(10),"<br>",print_r($inhalt,true));
						echo "<h3>Es ist ein Fehler aufgetreten:</h3>";
					}
					else
					//Es wurden Daten geliefert
					{
						//asort($data);
						foreach($data as $datarow => $datacontent){
							//echo "<br>".__file__." Zeile: ".__line__.": datarow=".$datarow." => datacontent=".str_replace(chr(10),"<br>",print_r($datacontent,true));
							echo "<br>".$datarow.";".$datacontent->feature.";<a href='$datacontent->uri'>".$datacontent->uri."</a>";
							if (isset($datacontent->properties->value)){
								echo "<br><b>value:".$datacontent->properties->value->value."</b>";
							}
							if (isset($datacontent->properties->status)){
								//echo "<br><b>status:".$datacontent->properties->status."</b>";
							}
							if (isset($datacontent->isEnabled)){
								echo "<br><b>status:".$datacontent->isEnabled."</b>";
							}
							if (isset($datacontent->isReady)){
								echo "<br><b>isReady:".$datacontent->isReady."</b>";
							}
							if (isset($datacontent->deviceId)){
								//echo "<br><b>deviceId:".$datacontent->deviceId."</b>";
							}


						}
					}
				}
			}
		}
	}

	/**
	 * api_listtemperaturen - Ausertung diverser Temperaturwerte
	 * @param GetURL - URL mit gewünschtem API-Aufruf
	 * @return api_liste string Array mit den jweiligen Bezeichnung des Feature und der notwendigne URL
	 */
	function api_read_all($getURL=""){
		if ($getURL == ""){
			$getURL='https://api.viessmann.com/iot/v1/equipment/installations/'.$this->installationId.'/gateways/'.$this->gatewaySerial.'/devices/'.$this->deviceId.'/features';
		}
		$erg = $this->api_curl_abruf($getURL);
		//echo "<br>".__file__."<br> Zeile: ".__line__.": ".$getURL."<br> erg=".str_replace(chr(10),"<br>",print_r($erg,true))."<br><br>";
		
		//API-Daten initialisieren
		$this->api_data = array();

		foreach($erg as $zeile => $inhalt){
			echo "<br>".__file__." Zeile: ".__line__.": Datenzeile ".$zeile;
			//echo "<br>".__file__." Zeile: ".__line__.": Inhalt".str_replace(chr(10),"<br>",print_r($inhalt,true));
			if (is_object($inhalt)){
				foreach($inhalt as $key => $data){
					//echo "<br>".__file__." Zeile: ".__line__.": key ".$key." => data=".str_replace(chr(10),"<br>",print_r($data,true));
					if ($key == "statusCode"){
						echo "<h2>Es ist ein Fehler aufgetreten:</h2>";
						echo "<a href=".$getURL."' target=new>".$getURL."</a>";
						echo "<br>".__file__." Zeile: ".__line__.": Inhalt".str_replace(chr(10),"<br>",print_r($inhalt,true));
						echo "<h3>Es ist ein Fehler aufgetreten:</h3>";
					}
					else
					//Es wurden Daten geliefert
					{
						foreach($data as $datarow => $datacontent){
							//echo "<br><b>".__file__." Zeile: ".__line__.":</b> datarow=".$datarow." => datacontent=".str_replace(chr(10),"<br>",print_r($datacontent,true));
							//echo "<br><b>".__file__." Zeile: ".__line__.":</b> datarow=".$datarow." => datacontent=".str_replace(chr(10),"<br>",print_r($datacontent->properties,true));
							switch ($datacontent->feature) {
								case "heating.sensors.temperature.outside":
									$this->api_data['aussen_temp'] = $datacontent->properties->value->value;
									break;

								case "heating.boiler.sensors.temperature.main":
									$this->api_data['heizung_temp_kessel'] = $datacontent->properties->value->value;
									break;

								case "heating.boiler.temperature":
									$this->api_data['heizung_temp_soll'] = $datacontent->properties->value->value;
									break;

								case "heating.burners.0":
									$this->api_data['brenner_status'] = $datacontent->properties->active->value;
									break;

								case "heating.burners.0.modulation":
									$this->api_data['brenner_modulation'] = $datacontent->properties->value->value;
									break;

								case "heating.burners.0.statistics":
									$this->api_data['brenner_stunden_gesamt'] = $datacontent->properties->hours->value;
									$this->api_data['brenner_starts_gesamt'] = $datacontent->properties->starts->value;
									break;

								case "heating.circuits.0.frostprotection":
									$this->api_data['ww_zirkulation'] = $datacontent->isEnabled;
									break;
								case "heating.circuits.1.circulation.pump":
									$this->api_data['heizung_zirkulation'] = $datacontent->isEnabled;
									break;
								case "heating.circuits.2.circulation.pump":
									$this->api_data['solar_zirkulation'] = $datacontent->isEnabled;
									break;
								case "heating.circuits.1.operating.programs.reduced":
									$this->api_data['heizung_absenkung_aktiv'] = $datacontent->properties->active->value;
									$this->api_data['heizung_absenkung_temp'] = $datacontent->properties->temperature->value;
									break;
								case "heating.circuits.1.operating.programs.normal":
								echo "<br><b>".__file__." Zeile: ".__line__.":</b> datarow=".$datarow." => datacontent=".str_replace(chr(10),"<br>",print_r($datacontent,true));
									$this->api_data['heizung_normal_aktiv'] = $datacontent->properties->active->value;
									$this->api_data['heizung_normal_temp'] = $datacontent->properties->temperature->value;
									break;
								case "heating.circuits.1.operating.programs.comfort":
									$this->api_data['heizung_comfort_aktiv'] = $datacontent->properties->active->value;
									$this->api_data['heizung_comfort_temp'] = $datacontent->properties->temperature->value;
									break;
								case "heating.circuits.1.operating.programs.external":
									$this->api_data['heizung_extern_aktiv'] = $datacontent->properties->active->value;
									$this->api_data['heizung_extern_temp'] = $datacontent->properties->temperature->value;
									break;
								case "heating.circuits.1.operating.programs.active":
									$this->api_data['heizung_programm_aktiv'] = $datacontent->properties->value->value;
									break;
								case "heating.circuits.1.operating.programs.holiday":
									$this->api_data['heizung_ferienEnabled'] = $datacontent->isEnabled;
									break;

								case "heating.circuits.1.frostprotection":
									$this->api_data['heizkreis_frostEnabled'] = $datacontent->isEnabled;
									break;
								case "heating.circuits.2.frostprotection":
									$this->api_data['solar_frostEnabled'] = $datacontent->isEnabled;
									break;
								case "heating.circuits.1.sensors.temperature.supply":
									$this->api_data['heizung_temp_vorlauf'] = $datacontent->properties->value->value;
									break;
								case "heating.dhw.temperature.main":
									$this->api_data['ww_temp_soll'] = $datacontent->properties->value->value;
									break;
								case "heating.dhw.sensors.temperature.dhwCylinder":
									$this->api_data['ww_temp_vorrat'] = $datacontent->properties->value->value;
									break;
								case "heating.solar.pumps.circuit":
									$this->api_data['solar_pumpe_Enabled'] = $datacontent->isEnabled;
									$this->api_data['solar_pumpe_Enabled_timestamp'] = $datacontent->timestamp;
									break;

								case "heating.circuits.2.circulation.pump":
									$this->api_data['solar_zirkulation'] = $datacontent->isEnabled;
									break;
								case "heating.solar.sensors.temperature.dhw":
									$this->api_data['solar_temp_ww'] = $datacontent->properties->value->value;
									break;
								case "heating.solar.sensors.temperature.collector":
									$this->api_data['solar_temp_koll'] = $datacontent->properties->value->value;
									break;
								case "heating.solar.power.cumulativeProduced":
									$this->api_data['solar_power_gesamt'] = $datacontent->properties->value->value;
									$this->api_data['solar_power_gesamt_unit'] = $datacontent->properties->value->unit;
									$this->api_data['solar_power_gesamt_timestamp'] = $datacontent->properties->timestamp;
									break;
								case "heating.solar.power.production":
									$this->api_data['solar_power_tage'] = $datacontent->properties->day->value;
									$this->api_data['solar_power_tage_unit'] = $datacontent->properties->day->unit;
									$this->api_data['solar_power_tage_read'] = $datacontent->properties->dayValueReadAt->value;
									break;

							}
						ksort($this->api_data);
						}
					}
				}
			}
		}
		echo "<br><b>".__file__." Zeile: ".__line__.":</b> this=".str_replace(chr(10),"<br>",print_r($this,true));
	}
}
?>
